/// QasTools: Desktop toolset for the Linux sound system ALSA.
/// \copyright See COPYING file.

#include "tray_mixer_icon.hpp"
#include "qastools_config.hpp"
#include <QAction>
#include <QEvent>
#include <QIcon>
#include <QWheelEvent>
#include <iostream>

#include <QProcess>
#include <qapplication.h>
#include <qprocess.h>
#include <qstringlist.h>

static const char * volume_xpm[] = {
"22 22 4 1",
" 	c None",
".	c #4D4D4D",
"+	c #4C4C4C",
"@	c #4E4E4E",
"                      ",
"          .+          ",
"         ...          ",
"        ....    .     ",
"        ....   ...    ",
"       .....    +..   ",
"      ......     ...  ",
" ...........  @.  ..  ",
" ...........  ..+  .. ",
" ...........   ..  .. ",
" ...........   ..  .. ",
" ...........   ..  .. ",
" ...........   ..  .. ",
" ...........  ...  .+ ",
" ...........  ..  +.  ",
"      ......     @..  ",
"       .....    ...   ",
"        ....   ...    ",
"        ....    .     ",
"         ...          ",
"          .+          ",
"                      "};

static const char * equalizer_xpm[] = {
"22 22 19 1",
" 	c None",
".	c #666666",
"+	c #000000",
"@	c #131313",
"#	c #2C2C2C",
"$	c #2B2B2B",
"%	c #2A2A2A",
"&	c #040404",
"*	c #FFFFFF",
"=	c #030303",
"-	c #1B1B1B",
";	c #1A1A1A",
">	c #0B0B0B",
",	c #181818",
"'	c #0A0A0A",
")	c #070707",
"!	c #050505",
"~	c #060606",
"{	c #020202",
"   .++.        .++.   ",
"   .++.        .++.   ",
"   .++.        .++.   ",
"   ++++        .++.   ",
"+@#$%%##&+     .++.   ",
"++******++     .++.   ",
"++......++     .++.   ",
"++++++++++     .++.   ",
"   ++++        .++.   ",
"   .++.        .++.   ",
"   .++.        ++++   ",
"   .++.     +=--;;-->+",
"   .++.     +,******++",
"   .++.     +'......++",
"   .++.     +)))!!~)){",
"   .++.        ++++   ",
"   .++.        .++.   ",
"   .++.        .++.   ",
"   .++.        .++.   ",
"   .++.        .++.   ",
"   .++.        .++.   ",
"   .++.        .++.   "};

static const char * app_xpm[] = {
/* columns rows colors chars-per-pixel */
"22 22 12 1 ",
"  c black",
". c #515151",
"X c #626262",
"o c #656565",
"O c #676767",
"+ c gray43",
"@ c #717171",
"# c #7C7C7C",
"$ c gray49",
"% c #7E7E7E",
"& c gray50",
"* c None",
/* pixels */
"**********************",
"**********************",
"**********************",
"**********************",
"*******%%%%%$*********",
"******%%%%%%%*********",
"******%%%%%%%&********",
"******$%%%%%%%********",
"******$%%%%%%%********",
"******@%%%%%%O********",
"*     X%%%%%%.       *",
"*     X%%%%%%.       *",
"*     X%%%%%%.       *",
"******+#%%%%$o********",
"*******%%%%&%*********",
"********%%%$**********",
"********%%%$**********",
"*********%$***********",
"**********************",
"**********************",
"**********************",
"**********************"
};


static const char * close_xpm[] = {
"22 22 45 1",
" 	c None",
".	c #000000",
"+	c #060606",
"@	c #0B0B0B",
"#	c #010101",
"$	c #131313",
"%	c #5B5B5B",
"&	c #6F6F6F",
"*	c #383838",
"=	c #020202",
"-	c #050505",
";	c #4E4E4E",
">	c #F4F4F4",
",	c #FFFFFF",
"'	c #C8C8C8",
")	c #0C0C0C",
"!	c #686868",
"~	c #F9F9F9",
"{	c #FCFCFC",
"]	c #CCCCCC",
"^	c #3E3E3E",
"/	c #2B2B2B",
"(	c #C6C6C6",
"_	c #FBFBFB",
":	c #CACACA",
"<	c #464646",
"[	c #454545",
"}	c #C9C9C9",
"|	c #C7C7C7",
"1	c #040404",
"2	c #E3E3E3",
"3	c #E1E1E1",
"4	c #FDFDFD",
"5	c #3B3B3B",
"6	c #424242",
"7	c #C1C1C1",
"8	c #FEFEFE",
"9	c #FAFAFA",
"0	c #C5C5C5",
"a	c #DDDDDD",
"b	c #E0E0E0",
"c	c #030303",
"d	c #CDCDCD",
"e	c #393939",
"f	c #707070",
"                      ",
"       ........       ",
"     ............     ",
"    .+@#......#@+.    ",
"   .$%&*=....=*&%$.   ",
"  .-;>,'*-..-*',>;+.  ",
"  .)!~,{]^==^]{,~!).  ",
"  ../({,_:<[}_,{|/..  ",
" ...1*({,{23_,4}51... ",
" ....167~,,8,90[1.... ",
" .....=6a8,,8b<c..... ",
" .....=6a8,,8b<c..... ",
" ....167~,,8,90[1.... ",
" ...1*({,{23_,4}51... ",
"  ../({,_:<<:_,{|/..  ",
"  .)!~,{d^==^d{,~!).  ",
"  .+;>,'e-..-e',>;+.  ",
"   .$%f*=....=*f%$.   ",
"    .+@#......#@+.    ",
"     ............     ",
"       ........       ",
"                      "};


Tray_Mixer_Icon::Tray_Mixer_Icon (
	QObject * parent_n ) :
QSystemTrayIcon ( parent_n )
{
	connect (
		this, SIGNAL ( activated ( QSystemTrayIcon::ActivationReason ) ),
		this, SLOT ( activation ( QSystemTrayIcon::ActivationReason ) ) );
	
	// Context menu
	{
		
		QAction * act_show ( new QAction ( this ) );
		act_show->setText ( tr ( "Volume" ) );
		static const QPixmap volumeXPM(volume_xpm);
		static const QIcon volumeIcon(volumeXPM);
		act_show->setIcon (QIcon(volumeIcon));
		
		QAction * act_EQ ( new QAction ( this ) );
		act_EQ->setText ( tr ( "Equalizer" ) );
		static const QPixmap equalizerXPM(equalizer_xpm);
		static const QIcon equalizerIcon(equalizerXPM);
		act_EQ->setIcon (QIcon(equalizerIcon));				

		QAction * act_MIX ( new QAction ( this ) );
		act_MIX->setText ( tr ( "App Mixer" ) );
		static const QPixmap appXPM(app_xpm);
		static const QIcon appIcon(appXPM);
		act_MIX->setIcon (QIcon(appIcon));				

		QAction * act_quit ( new QAction ( this ) );
		act_quit->setText ( tr ( "Hide Volume" ) );
		act_quit->setShortcut ( QKeySequence ( QKeySequence::Quit ) );
		static const QPixmap closeXPM(close_xpm);
		static const QIcon closeIcon(closeXPM);
		act_quit->setIcon (QIcon(closeIcon));		

		connect ( act_show, SIGNAL ( triggered() ),
			this, SIGNAL ( sig_activated() ) );
			
		connect(act_EQ, SIGNAL( triggered() ),
            this, SLOT ( show_EQ() ));

		connect(act_MIX, SIGNAL( triggered() ),
            this, SLOT ( show_MIXER() ));
			
		connect ( act_quit, SIGNAL ( triggered() ),
			this, SIGNAL ( sig_quit() ) );

		_cmenu.addAction ( act_show );
		_cmenu.addAction ( act_EQ );
		_cmenu.addAction ( act_MIX );
		_cmenu.addAction ( act_quit );
		setContextMenu ( &_cmenu );
	}
}


void
Tray_Mixer_Icon::activation ( QSystemTrayIcon::ActivationReason reason_n )
{
  //::std::cout << "Tray_Mixer::activation " << reason_n << "\n";

  if ( reason_n == QSystemTrayIcon::Context ) {
    return;
  } else if ( reason_n == QSystemTrayIcon::MiddleClick ) {
    emit sig_middle_click ();
  } else {
    emit sig_activated ();
  }
}

bool
Tray_Mixer_Icon::event ( QEvent * event_n )
{
  //::std::cout << "Tray_Mixer_Icon::event: " << event_n->type() << "\n";

  bool res ( QSystemTrayIcon::event ( event_n ) );

  switch ( event_n->type () ) {
  case QEvent::Wheel: {
    QWheelEvent * wev ( static_cast< QWheelEvent * > ( event_n ) );
    emit sig_wheel_delta ( wev->delta () );
    res = true;
  } break;
  case QEvent::ToolTip:
    emit sig_hover ();
    break;
  default:
    break;
  }

  return res;
}

void
Tray_Mixer_Icon::show_EQ()
{
	QProcess process;
	process.startDetached("/usr/bin/pulseaudio-equalizer-gtk", QStringList()<< "oooo");	
}

void
Tray_Mixer_Icon::show_MIXER()
{
	QProcess p;
	p.startDetached("/usr/bin/pavucontrol", QStringList()<< "oooo");	
}
